package org.globus.transfer.reliable.service.test.client;

import junit.framework.TestCase;
import javax.xml.namespace.QName;
import org.globus.rft.generated.TransferType;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.transfer.reliable.service.exception.RftException;
import org.globus.transfer.reliable.service.RFTResourceManager;
import org.globus.transfer.reliable.service.client.ClientFactory;

public class ClientFactoryTest extends TestCase
{
	private int requestId;
	private int transferId;
	private int concurrency;
	private ResourcePropertySet rpSet;
	private RFTResourceManager resourceManager;
	private TransferType transferJob;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		this.requestId = 42;
		this.transferId = 42;
		this.concurrency = 42;
		String namespaceURI = "http://www.globus.org/test";
		String localName = "ClientFactoryTest";
		QName myQName= new QName(namespaceURI,localName);
		this.rpSet = new SimpleResourcePropertySet(myQName);
		this.resourceManager
		    = new RFTResourceManager(
		        this.requestId, this.concurrency, this.rpSet);
		this.transferJob = new TransferType();
	}

	/**
	 * Test for the helper function getGlobusURL(String)
	 */ 
	public void testGetGlobusUrl()
	{
		String[] failureStrings = new String[5];
		String[] okStrings = new String[1];
		failureStrings[0] = null;
		failureStrings[1] = "";
		failureStrings[2] = "globus";
		failureStrings[3] = "http://";
	    failureStrings[4] = "http://www.globus.org/";
	    okStrings[0] = "gsiftp://www.globus.org:2811/myFile";
	    
	    // check strings that should cause failures
	    for(int i=0; i<failureStrings.length; i++) {
	        try {
	            ClientFactory.getGlobusURL(failureStrings[i]);
                fail("An exception should be thrown for url string " +
                     failureStrings[i]);
	        } catch (RftException e1) {
	    	    // this is what we expect
	        } catch (Exception e2) {
	    	    fail("An RftException should be thrown for url string " +
	    	         failureStrings[i]);
	        }
	    }
	    
        // check strings that should work
	    for(int i=0; i<okStrings.length; i++) {
	        try {
	            ClientFactory.getGlobusURL(okStrings[i]);
	        } catch (Exception e) {
		    	fail(okStrings[i] + " should have been ok for " +
		             "creating a GlobusURL");
	        }
	    }
	}
	
    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateDeleteClient()
	{
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createDeleteClient(transferJob, null);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // TransferType parameter is null
	    try {
	    	ClientFactory.createDeleteClient(null, resourceManager);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	}

    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateGetPutTransferClient()
	{	
		// TransferType parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            null, resourceManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            transferJob, null, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // ResourcePropertySet parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            transferJob, resourceManager, null, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	}

    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateThirdPartyTransferClient()
	{
		// TransferType parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            null, resourceManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            transferJob, null, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // ResourcePropertySet parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            transferJob, resourceManager, null, transferId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	}

	public void testCreateURLExpanderClient()
	{
		// TransferType parameter is null
	    try {
	    	ClientFactory.createURLExpanderClient(
	            null, resourceManager, requestId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	    
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createURLExpanderClient(
	            transferJob, null, requestId);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e1) {
	        // that's what we expect here
	    } catch (Exception e2) {
	        fail("should be an RftException here");
	    }
	}

}
