package org.globus.transfer.reliable.service.test.connection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

import org.globus.transfer.reliable.service.RFTResourceManager;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;

import org.globus.transfer.reliable.client.BaseRFTClient;

import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionCacheImpl;


public class ConnectionCacheImplTest extends TestCase
{
    public static int connType = -1;

    public static Connection conn = null;
    public static ConnectionManager connManager = null;
    public static ConnectionParameter params = null;
    public static ConnectionCacheImpl ccImpl = null;

    public static EndpointReferenceType epr = null;
    public static GSSCredential credential = null;

    public static TransferRequestType transferReq = null;

    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    public ConnectionCacheImplTest(String name)
    {
        super(name);
    }

    public static void setOptions()
    {
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception
    {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        baseRequestType = ReliableFileTransferClient.getTransfer(
            reqFile, credEPR);
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        ConnectionCacheImplTest.epr = createRFT(TRANSFER);

        ConnectionCacheImplTest.transferReq = TestConfig.getSingleTransfer();

        ConnectionCacheImplTest.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        TestConfig.printConfig();

        System.out.println(
            "setUp " + (((ConnectionCacheImplTest.epr == null) ||
                         (ConnectionCacheImplTest.transferReq == null) ||
                         (ConnectionCacheImplTest.credential == null)) ? "FAILED" : "OK"));
    }

    public void testConnectionCacheImpl()
    {
        try
        {
            ConnectionCacheImplTest.connType = ConnectionManager.SINGLE_CONNECTION;
            ConnectionCacheImplTest.params = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                TestConfig.sourcePath, TestConfig.destinationHost,
                TestConfig.destPort, TestConfig.destPath, TestConfig.subject);
            ConnectionCacheImplTest.connManager = ConnectionManager.getInstance();

            try
            {
                System.out.println("ConnectionCacheImplTest.params: " + ConnectionCacheImplTest.params);
                System.out.println("ConnectionCacheImplTest.connManager: " + ConnectionCacheImplTest.connManager);

                ConnectionCacheImplTest.conn = ConnectionCacheImplTest.connManager.getConnection(
                    ConnectionCacheImplTest.connType, ConnectionCacheImplTest.params,
                    ConnectionCacheImplTest.transferReq.getRftOptions(),
                    ConnectionCacheImplTest.credential, false);

                System.out.println("ConnectionCacheImplTest.conn: " + ConnectionCacheImplTest.conn);
                if (ConnectionCacheImplTest.conn == null)
                {
                    fail("Cannot create Connection object");
                }
            }
            catch(Exception e)
            {
                fail("Failed to create Connection: " + e);
            }

            ConnectionCacheImplTest.ccImpl = new ConnectionCacheImpl(
                ConnectionCacheImplTest.connType, 5000);
            if (ConnectionCacheImplTest.ccImpl == null)
            {
                fail("Cannot create ConnectionCacheImpl object");
            }
        }
        catch(Exception e)
        {
            fail("Failed to create ConnectionCacheImpl: " + e);
        }
    }

    public void testAddConnection()
    {
        try
        {
            ConnectionCacheImplTest.ccImpl.addConnection(ConnectionCacheImplTest.conn);
        }
        catch(Exception e)
        {
            fail("Failed to addConnection: " + e);
        }
    }

    public void testGetConnection()
    {
        try
        {
            Connection conn = ConnectionCacheImplTest.ccImpl.getConnection(
                ConnectionCacheImplTest.params);
            if (conn != ConnectionCacheImplTest.conn)
            {
                fail("Retrieved connection does not match original connection!");
            }
        }
        catch(Exception e)
        {
            fail("Failed to get connection: " + e);
        }
    }

    public void testShrink()
    { 
        try
        {
            ConnectionCacheImplTest.ccImpl.shrink();
        }
        catch(Exception e)
        {
            fail("Failed to shrink connection cache: " + e);
        }
    }

    public void testGetType()
    {
        try
        {
            int type = ConnectionCacheImplTest.ccImpl.getType();
            if (type != ConnectionCacheImplTest.connType)
            {
                fail("Retrieved cache type does not match original cache type!");
            }
        }
        catch(Exception e)
        {
            fail("Failed to get connection: " + e);
        }
    }

    public void testGetOldestTimestamp()
    {
        try
        {
            long ts = ConnectionCacheImplTest.ccImpl.getOldestTimestamp();
            System.out.println("Oldest timestamp is: " + ts);
        }
        catch(Exception e)
        {
            fail("Failed to retrieve the oldest timestamp: " + e);
        }
    }

    public void testCloseAllConnections()
    {
        try
        {
            ConnectionCacheImplTest.ccImpl.closeAllConnections();
        }
        catch(Exception e)
        {
            fail("Failed to close all connections: " + e);
        }
    }

    public void testRemoveExpiredConnections()
    {
        try
        {
            /* make sure the connection is expired */
            Thread.sleep(5000);

            ConnectionCacheImplTest.ccImpl.removeExpiredConnections();

            /* this connection should not be found */
            Connection conn = ConnectionCacheImplTest.ccImpl.getConnection(
                ConnectionCacheImplTest.params);
            if (conn == ConnectionCacheImplTest.conn)
            {
                fail("Connection retrieved that should have been expired!");
            }
        }
        catch(Exception e)
        {
            fail("Failed to close all connections: " + e);
        }
    }
}
