package org.globus.transfer.reliable.service.test.connection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

import org.globus.transfer.reliable.service.RFTResourceManager;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;

import org.globus.transfer.reliable.client.BaseRFTClient;

import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;


public class ConnectionParameterTest extends TestCase
{
    public static ConnectionParameter params = null;

    public static EndpointReferenceType epr = null;
    public static GSSCredential credential = null;

    public static TransferRequestType transferReq = null;

    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    public static ConnectionParameter cp = null;

    public static void setOptions()
    {
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception
    {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        baseRequestType = ReliableFileTransferClient.getTransfer(
            reqFile, credEPR);
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        this.epr = createRFT(TRANSFER);

        this.transferReq = TestConfig.getSingleTransfer();

        this.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        TestConfig.printConfig();

        System.out.println("setUp " +
                           (((this.epr == null) || (this.transferReq == null) ||
                             (this.credential == null)) ? "FAILED" : "OK"));

        try
        {
            ConnectionParameterTest.cp = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                TestConfig.sourcePath, TestConfig.destinationHost,
                TestConfig.destPort, TestConfig.destPath, TestConfig.subject);
        }
        catch(Exception e)
        {
            fail("Failed to create connection parameters: " + e);
        }
    }

    public void testGetType()
    {
        try
        {
            ConnectionParameterTest.cp.getType();
        }
        catch(Exception e)
        {
            fail("Failed to get Type: " + e);
        }
    }

    public void testGetHashCode()
    {
        try
        {
            ConnectionParameterTest.cp.getHashCode();
        }
        catch(Exception e)
        {
            fail("Failed to get Hash code: " + e);
        }
    }

    public void testGetDestinationHost()
    {
        try
        {
            ConnectionParameterTest.cp.getDestinationHost();
        }
        catch(Exception e)
        {
            fail("Failed to get destination host: " + e);
        }
    }

    public void testGetDestinationPath()
    {
        try
        {
            ConnectionParameterTest.cp.getDestinationPath();
        }
        catch(Exception e)
        {
            fail("Failed to get destination path: " + e);
        }
    }

    public void testGetDestinationPort()
    {
        try
        {
            ConnectionParameterTest.cp.getDestinationPort();
        }
        catch(Exception e)
        {
            fail("Failed to get destination port: " + e);
        }
    }

    public void testGetSourceHost()
    {
        try
        {
            ConnectionParameterTest.cp.getSourceHost();
        }
        catch(Exception e)
        {
            fail("Failed to get source host: " + e);
        }
    }

    public void testGetSourcePath()
    {
        try
        {
            ConnectionParameterTest.cp.getSourcePath();
        }
        catch(Exception e)
        {
            fail("Failed to get source path: " + e);
        }
    }

    public void testGetSourcePort()
    {
        try
        {
            ConnectionParameterTest.cp.getSourcePort();
        }
        catch(Exception e)
        {
            fail("Failed to get source port: " + e);
        }
    }

    public void testGetUserDN()
    {
        try
        {
            ConnectionParameterTest.cp.getUserDN();
        }
        catch(Exception e)
        {
            fail("Failed to get user DN: " + e);
        }
    }

}
