package org.globus.transfer.reliable.service.test.connection;

import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.globus.ftp.ByteRangeList;
import org.globus.ftp.DataChannelAuthentication;
import org.globus.ftp.GridFTPClient;
import org.globus.ftp.GridFTPRestartMarker;
import org.globus.ftp.MlsxEntry;
import org.globus.ftp.RetrieveOptions;
import org.globus.ftp.Session;
import org.globus.ftp.exception.ServerException;
import org.globus.ftp.vanilla.Reply;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

import org.globus.transfer.reliable.service.RFTResourceManager;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;

import org.globus.transfer.reliable.client.BaseRFTClient;

import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionCacheImpl;
import org.globus.transfer.reliable.service.cache.MlsdConnectionImpl;


public class MlsdConnectionImplTest extends TestCase
{
    public static int connType = -1;

    public static Connection conn = null;
    public static ConnectionManager connManager = null;
    public static ConnectionParameter params = null;
    public static MlsdConnectionImpl mcImpl = null;

    public static EndpointReferenceType epr = null;
    public static GSSCredential credential = null;

    public static TransferRequestType transferReq = null;

    public static String TESTDIR = "SCIT_TESTDIR";

    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    public MlsdConnectionImplTest(String name)
    {
        super(name);
    }

    public static void setOptions()
    {
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception
    {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        baseRequestType = ReliableFileTransferClient.getTransfer(
            reqFile, credEPR);
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        MlsdConnectionImplTest.epr = createRFT(TRANSFER);

        MlsdConnectionImplTest.transferReq = TestConfig.getSingleTransfer();

        MlsdConnectionImplTest.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        TestConfig.printConfig();

        System.out.println(
            "setUp " + (((MlsdConnectionImplTest.epr == null) ||
                         (MlsdConnectionImplTest.transferReq == null) ||
                         (MlsdConnectionImplTest.credential == null)) ? "FAILED" : "OK"));
    }

    public void testMlsdConnectionImpl()
    {
        try
        {
            MlsdConnectionImplTest.connType = ConnectionManager.MLSD_CONNECTION;
            MlsdConnectionImplTest.params = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                MlsdConnectionImplTest.TESTDIR, TestConfig.destinationHost,
                TestConfig.destPort, TestConfig.destPath, TestConfig.subject);
            MlsdConnectionImplTest.connManager = ConnectionManager.getInstance();

            try
            {
                System.out.println("MlsdConnectionImplTest.params: " + MlsdConnectionImplTest.params);
                System.out.println("MlsdConnectionImplTest.connManager: " + MlsdConnectionImplTest.connManager);

                MlsdConnectionImplTest.conn = MlsdConnectionImplTest.connManager.getConnection(
                    MlsdConnectionImplTest.connType, MlsdConnectionImplTest.params,
                    MlsdConnectionImplTest.transferReq.getRftOptions(),
                    MlsdConnectionImplTest.credential, false);

                System.out.println("MlsdConnectionImplTest.conn: " + MlsdConnectionImplTest.conn);
                if (MlsdConnectionImplTest.conn == null)
                {
                    fail("Cannot create Connection object");
                }
            }
            catch(Exception e)
            {
                fail("Failed to create Connection: " + e);
            }

            MlsdConnectionImplTest.mcImpl = new MlsdConnectionImpl(
                MlsdConnectionImplTest.params,
                MlsdConnectionImplTest.transferReq.getRftOptions(),
                MlsdConnectionImplTest.credential);
            if (MlsdConnectionImplTest.mcImpl == null)
            {
                fail("Cannot create MlsdConnectionImpl object");
            }
        }
        catch(Exception e)
        {
            fail("Failed to create MlsdConnectionImpl: " + e);
        }
    }


    public void testMakeDirectory()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.makeDirectory(
                MlsdConnectionImplTest.TESTDIR);
        }
        catch(Exception e)
        {
            fail("Failed to create directory: " + e);
        }
    }

    public void testChangeDirectory()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.changeDirectory(
                MlsdConnectionImplTest.TESTDIR, false);
        }
        catch(Exception e)
        {
            fail("Failed to change directory: " + e);
        }
    }

    public void testDoMlsd()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.setPath("/");
            Vector entries = MlsdConnectionImplTest.mcImpl.doMlsd("test");
            System.out.println("GOT ENTRIES: " + entries);
        }
        catch(Exception e)
        {
            fail("Failed to do mlsd: " + e);
        }
    }

    public void testDoMlst()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.setPath("/");
            MlsxEntry mEntry = MlsdConnectionImplTest.mcImpl.doMlst();
            System.out.println("GOT PERMISSIONS: " + mEntry);
        }
        catch(Exception e)
        {
            fail("Failed to do mlst: " + e);
        }
    }

    public void testDelete()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.setPath(MlsdConnectionImplTest.TESTDIR);
            MlsdConnectionImplTest.mcImpl.deleteDirectory();

            try
            {
                MlsdConnectionImplTest.mcImpl.changeDirectory(
                    MlsdConnectionImplTest.TESTDIR, false);

                fail("Changed directory to a dir that has been deleted!");
            }
            catch(Exception e)
            {
                /* this should happen since the dir should be gone  */
            }
        }
        catch(Exception e)
        {
            fail("Failed to delete directory: " + e);
        }
    }

    public void testCheckConnectionIntegrity()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.checkConnectionIntegrity();
        }
        catch(Exception e)
        {
            fail("Failed to set check connection integrity");
        }
    }

    public void testCheckForReuse()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.checkForReuse(
                MlsdConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to set check for re-use");
        }
    }

    public void testCloseConnection()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.closeConnection();
        }
        catch(Exception e)
        {
            fail("Failed to set Restart Marker");
        }
    }

    public void testIsDcau()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.isDcau();
        }
        catch(Exception e)
        {
            fail("Failed to check is Dcau");
        }
    }

    public void testGetType()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.getType();
        }
        catch(Exception e)
        {
            fail("Failed to check type");
        }
    }

    public void testGetNumberFDs()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.getNumberFDs();
        }
        catch(Exception e)
        {
            fail("Failed to get Number FDs");
        }
    }

    public void testGetLastUsed()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.getLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to get last used");
        }
    }

    public void testGetPath()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.getPath();
        }
        catch(Exception e)
        {
            fail("Failed to get path");
        }
    }

    public void testSetPath()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.setPath("/");
        }
        catch(Exception e)
        {
            fail("Failed to set path");
        }
    }

    public void testUpdateLastUsed()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.updateLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to update last used");
        }
    }

    public void testUpdateForReuse()
    {
        try
        {
            MlsdConnectionImplTest.mcImpl.updateForReuse(
                MlsdConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to update for re-use");
        }
    }
}
