package org.globus.transfer.reliable.service.test.connection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.globus.ftp.ByteRangeList;
import org.globus.ftp.DataChannelAuthentication;
import org.globus.ftp.GridFTPClient;
import org.globus.ftp.GridFTPRestartMarker;
import org.globus.ftp.MlsxEntry;
import org.globus.ftp.RetrieveOptions;
import org.globus.ftp.Session;
import org.globus.ftp.exception.ServerException;
import org.globus.ftp.vanilla.Reply;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

import org.globus.transfer.reliable.service.RFTResourceManager;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;

import org.globus.transfer.reliable.client.BaseRFTClient;

import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionCacheImpl;
import org.globus.transfer.reliable.service.cache.SingleConnectionImpl;


public class SingleConnectionImplTest extends TestCase
{
    public static int connType = -1;

    public static Connection conn = null;
    public static ConnectionManager connManager = null;
    public static ConnectionParameter params = null;
    public static SingleConnectionImpl scImpl = null;

    public static EndpointReferenceType epr = null;
    public static GSSCredential credential = null;

    public static TransferRequestType transferReq = null;

    public static String TESTDIR = "SCIT_TESTDIR";

    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    public SingleConnectionImplTest(String name)
    {
        super(name);
    }

    public static void setOptions()
    {
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception
    {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        baseRequestType = ReliableFileTransferClient.getTransfer(
            reqFile, credEPR);
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        SingleConnectionImplTest.epr = createRFT(TRANSFER);

        SingleConnectionImplTest.transferReq = TestConfig.getSingleTransfer();

        SingleConnectionImplTest.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        TestConfig.printConfig();

        System.out.println(
            "setUp " + (((SingleConnectionImplTest.epr == null) ||
                         (SingleConnectionImplTest.transferReq == null) ||
                         (SingleConnectionImplTest.credential == null)) ? "FAILED" : "OK"));
    }

    public void testSingleConnectionImpl()
    {
        try
        {
            SingleConnectionImplTest.connType = ConnectionManager.SINGLE_CONNECTION;
            SingleConnectionImplTest.params = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                SingleConnectionImplTest.TESTDIR, TestConfig.destinationHost,
                TestConfig.destPort, TestConfig.destPath, TestConfig.subject);
            SingleConnectionImplTest.connManager = ConnectionManager.getInstance();

            try
            {
                System.out.println("SingleConnectionImplTest.params: " + SingleConnectionImplTest.params);
                System.out.println("SingleConnectionImplTest.connManager: " + SingleConnectionImplTest.connManager);

                SingleConnectionImplTest.conn = SingleConnectionImplTest.connManager.getConnection(
                    SingleConnectionImplTest.connType, SingleConnectionImplTest.params,
                    SingleConnectionImplTest.transferReq.getRftOptions(),
                    SingleConnectionImplTest.credential, false);

                System.out.println("SingleConnectionImplTest.conn: " + SingleConnectionImplTest.conn);
                if (SingleConnectionImplTest.conn == null)
                {
                    fail("Cannot create Connection object");
                }
            }
            catch(Exception e)
            {
                fail("Failed to create Connection: " + e);
            }

            SingleConnectionImplTest.scImpl = new SingleConnectionImpl(
                SingleConnectionImplTest.params,
                SingleConnectionImplTest.transferReq.getRftOptions(),
                SingleConnectionImplTest.credential);
            if (SingleConnectionImplTest.scImpl == null)
            {
                fail("Cannot create SingleConnectionImpl object");
            }
        }
        catch(Exception e)
        {
            fail("Failed to create SingleConnectionImpl: " + e);
        }
    }


    public void testMakeDirectory()
    {
        try
        {
            SingleConnectionImplTest.scImpl.makeDirectory(
                SingleConnectionImplTest.TESTDIR);
        }
        catch(Exception e)
        {
            fail("Failed to create directory: " + e);
        }
    }

    public void testChangeDirectory()
    {
        try
        {
            SingleConnectionImplTest.scImpl.changeDirectory(
                SingleConnectionImplTest.TESTDIR, false);
        }
        catch(Exception e)
        {
            fail("Failed to change directory: " + e);
        }
    }

    public void testSetPermissions()
    {
        try
        {
            SingleConnectionImplTest.scImpl.setPermissions("0755");
        }
        catch(Exception e)
        {
            fail("Failed to set directory permissions: " + e);
        }
    }

    public void testDoMlst()
    {
        try
        {
            SingleConnectionImplTest.scImpl.setPath("/");
            MlsxEntry mEntry = SingleConnectionImplTest.scImpl.doMlst();
            System.out.println("GOT PERMISSIONS: " + mEntry);
        }
        catch(Exception e)
        {
            fail("Failed to get directory listing: " + e);
        }
    }

    public void testDelete()
    {
        try
        {
            SingleConnectionImplTest.scImpl.setPath(SingleConnectionImplTest.TESTDIR);
            SingleConnectionImplTest.scImpl.deleteDirectory();

            try
            {
                SingleConnectionImplTest.scImpl.changeDirectory(
                    SingleConnectionImplTest.TESTDIR, false);

                fail("Changed directory to a dir that has been deleted!");
            }
            catch(Exception e)
            {
                /* this should happen since the dir should be gone  */
            }
        }
        catch(Exception e)
        {
            fail("Failed to delete directory: " + e);
        }
    }

    public void testSetOptionsRFTOptionsType()
    {
        try
        {
            if (SingleConnectionImplTest.transferReq != null)
            {
                RFTOptionsType rftOptions =
                    SingleConnectionImplTest.transferReq.getRftOptions();
                if (rftOptions != null)
                {
                    rftOptions.setParallelStreams(new Integer(0));
                    SingleConnectionImplTest.scImpl.setOptions(rftOptions);
                }
            }
        }
        catch(Exception e)
        {
            fail("Failed to set RFT options");
        }
    }

    public void testSetRestartMarkerString()
    {
        try
        {
            SingleConnectionImplTest.scImpl.setRestartMarker("1");
        }
        catch(Exception e)
        {
            fail("Failed to set Restart Marker: " + e);
        }
    }

    public void testCheckConnectionIntegrity()
    {
        try
        {
            SingleConnectionImplTest.scImpl.checkConnectionIntegrity();
        }
        catch(Exception e)
        {
            fail("Failed to set check connection integrity: " + e);
        }
    }

    public void testCheckForReuse()
    {
        try
        {
            SingleConnectionImplTest.scImpl.checkForReuse(
                SingleConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to set check for re-use: " + e);
        }
    }

    public void testCloseConnection()
    {
        try
        {
            SingleConnectionImplTest.scImpl.closeConnection();
        }
        catch(Exception e)
        {
            fail("Failed to set close Connection: " + e);
        }
    }

    public void testIsDcau()
    {
        try
        {
            SingleConnectionImplTest.scImpl.isDcau();
        }
        catch(Exception e)
        {
            fail("Failed to check is Dcau: " + e);
        }
    }

    public void testGetType()
    {
        try
        {
            SingleConnectionImplTest.scImpl.getType();
        }
        catch(Exception e)
        {
            fail("Failed to check type: " + e);
        }
    }

    public void testGetNumberFDs()
    {
        try
        {
            SingleConnectionImplTest.scImpl.getNumberFDs();
        }
        catch(Exception e)
        {
            fail("Failed to get Number FDs: " + e);
        }
    }

    public void testGetLastUsed()
    {
        try
        {
            SingleConnectionImplTest.scImpl.getLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to get last used: " + e);
        }
    }

    public void testGetPath()
    {
        try
        {
            SingleConnectionImplTest.scImpl.getPath();
        }
        catch(Exception e)
        {
            fail("Failed to get path: " + e);
        }
    }

    public void testSetPath()
    {
        try
        {
            SingleConnectionImplTest.scImpl.setPath("/");
        }
        catch(Exception e)
        {
            fail("Failed to set path: " + e);
        }
    }

    public void testUpdateLastUsed()
    {
        try
        {
            SingleConnectionImplTest.scImpl.updateLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to update last used: " + e);
        }
    }

    public void testUpdateForReuse()
    {
        try
        {
            SingleConnectionImplTest.scImpl.updateForReuse(
                SingleConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to update for re-use: " + e);
        }
    }
}
