package org.globus.transfer.reliable.service.test.connection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

import org.globus.transfer.reliable.service.RFTResourceManager;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;

import org.globus.transfer.reliable.client.BaseRFTClient;

import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionCacheImpl;
import org.globus.transfer.reliable.service.cache.ThirdPartyConnectionImpl;


public class ThirdPartyConnectionImplTest extends TestCase
{
    public static int connType = -1;

    public static Connection conn = null;
    public static ConnectionManager connManager = null;
    public static ConnectionParameter params = null;
    public static ThirdPartyConnectionImpl tpcImpl = null;

    public static EndpointReferenceType epr = null;
    public static GSSCredential credential = null;

    public static TransferRequestType transferReq = null;

    public static String TESTDIR = "SCIT_TESTDIR";

    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    public ThirdPartyConnectionImplTest(String name)
    {
        super(name);
    }

    public static void setOptions()
    {
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception
    {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        baseRequestType = ReliableFileTransferClient.getTransfer(
            reqFile, credEPR);
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        ThirdPartyConnectionImplTest.epr = createRFT(TRANSFER);

        ThirdPartyConnectionImplTest.transferReq = TestConfig.getSingleTransfer();

        ThirdPartyConnectionImplTest.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        TestConfig.printConfig();

        System.out.println(
            "setUp " + (((ThirdPartyConnectionImplTest.epr == null) ||
                         (ThirdPartyConnectionImplTest.transferReq == null) ||
                         (ThirdPartyConnectionImplTest.credential == null)) ? "FAILED" : "OK"));
    }

    public void testThirdPartyConnectionImpl()
    {
        try
        {
            ThirdPartyConnectionImplTest.connType = ConnectionManager.THIRDPARTY_CONNECTION;
            ThirdPartyConnectionImplTest.params = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                ThirdPartyConnectionImplTest.TESTDIR, TestConfig.destinationHost,
                TestConfig.destPort, TestConfig.destPath, TestConfig.subject);
            ThirdPartyConnectionImplTest.connManager = ConnectionManager.getInstance();

            try
            {
                ThirdPartyConnectionImplTest.conn = ThirdPartyConnectionImplTest.connManager.getConnection(
                    ThirdPartyConnectionImplTest.connType, ThirdPartyConnectionImplTest.params,
                    ThirdPartyConnectionImplTest.transferReq.getRftOptions(),
                    ThirdPartyConnectionImplTest.credential, false);

                if (ThirdPartyConnectionImplTest.conn == null)
                {
                    fail("Cannot create Connection object");
                }
            }
            catch(Exception e)
            {
                fail("Failed to create Connection: " + e);
            }

            ThirdPartyConnectionImplTest.tpcImpl = new ThirdPartyConnectionImpl(
                ThirdPartyConnectionImplTest.params,
                ThirdPartyConnectionImplTest.transferReq.getRftOptions(),
                ThirdPartyConnectionImplTest.credential);
            if (ThirdPartyConnectionImplTest.tpcImpl == null)
            {
                fail("Cannot create ThirdPartyConnectionImpl object");
            }
        }
        catch(Exception e)
        {
            fail("Failed to create ThirdPartyConnectionImpl: " + e);
        }
    }

    public void testIsDcau()
    {
        try
        {
            boolean isDcau = ThirdPartyConnectionImplTest.tpcImpl.isDcau();
            System.out.println("Is DCAU: " + isDcau);
        }
        catch(Exception e)
        {
            fail("Failed to test isDcau: " + e);
        }
    }

    public void testSetRestartMarker()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.setRestartMarker("FOOBAR");
        }
        catch(Exception e)
        {
            fail("Failed to set Restart Marker: " + e);
        }
    }

    public void testCloseConnection()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.closeConnection();
        }
        catch(Exception e)
        {
            fail("Failed to set close Connection: " + e);
        }
    }

    public void testCheckForReuse()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.checkForReuse(
                ThirdPartyConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to check for re-use: " + e);
        }
    }

    public void testCheckConnectionIntegrity()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.checkConnectionIntegrity();
        }
        catch(Exception e)
        {
            fail("Failed to check connection integrity: " + e);
        }
    }

    public void testUpdateForReuse()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.updateForReuse(
                ThirdPartyConnectionImplTest.params);
        }
        catch(Exception e)
        {
            fail("Failed to update for re-use: " + e);
        }
    }

    public void testNormalExtendedTransfer()
    {
        fail("Not yet implemented");
    }

    public void testNormalNonExtendedTransfer()
    {
        fail("Not yet implemented");
    }

    public void testDoMlstOnSource()
    {
        fail("Not yet implemented");
    }

    public void testSetPermissionsOnDestination()
    {
        fail("Not yet implemented");
    }

    public void testSetOptions()
    {
        try
        {
            if (ThirdPartyConnectionImplTest.transferReq != null)
            {
                RFTOptionsType rftOptions =
                    ThirdPartyConnectionImplTest.transferReq.getRftOptions();
                if (rftOptions != null)
                {
                    rftOptions.setParallelStreams(new Integer(0));
                    ThirdPartyConnectionImplTest.tpcImpl.setOptions(rftOptions);
                }
            }
        }
        catch(Exception e)
        {
            fail("Failed to set RFT options");
        }
    }

    public void testGetType()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.getType();
        }
        catch(Exception e)
        {
            fail("Failed to get type: " + e);
        }
    }

    public void testGetNumberFDs()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.getNumberFDs();
        }
        catch(Exception e)
        {
            fail("Failed to get Number of FDs: " + e);
        }
    }

    public void testGetLastUsed()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.getLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to get last used: " + e);
        }
    }

    public void testGetSourcePath()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.getSourcePath();
        }
        catch(Exception e)
        {
            fail("Failed to get source path: " + e);
        }
    }

    public void testGetDestinationPath()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.getDestinationPath();
        }
        catch(Exception e)
        {
            fail("Failed to get destination path: " + e);
        }
    }

    public void testUpdateLastUsed()
    {
        try
        {
            ThirdPartyConnectionImplTest.tpcImpl.updateLastUsed();
        }
        catch(Exception e)
        {
            fail("Failed to get update last used: " + e);
        }
    }

}
